from scvf.constants import settings_keys


def pipeline_loop(image_container, pipeline_manager, settings, output_consumer, running):
    while running():
        if settings.is_updated():
            pipeline_manager.set_pipeline(settings.get(settings_keys["pipeline_name"], ''))
        img = image_container.get()
        if img is not None:
            pipeline_manager.process(img)
            output_consumer(pipeline_manager.get_output())



class PipelineManager:
    def __init__(self, pipelines):
        self.pipelines = pipelines
        if len(pipelines.keys()) < 1:
            self.current_pipeline = ""
        else:
            self.current_pipeline = list(pipelines.keys())[0]

    def set_pipeline(self, pipeline_id):
        if pipeline_id in self.pipelines.keys():
            self.current_pipeline = pipeline_id

    def process(self, img):
        pipeline = self.pipelines.get(self.current_pipeline, None)
        if pipeline is not None:
            pipeline.process(img)

    def get_output(self):
        pipeline = self.pipelines.get(self.current_pipeline, None)
        if pipeline is not None:
            return pipeline.get_output()



from scvf.constants import settings_keys


def camera_loop(image_container, camera_manager, settings, running):
    while running():
        if settings.is_updated():
            camera_manager.set_exposure(settings.get(settings_keys["exposure"], ""))
            camera_manager.set_camera(settings.get(settings_keys["camera_id"], ""))

        img = camera_manager.get_image()
        if img is not None:
            image_container.set(img)
    camera_manager.release()


class Settings(dict):
    def __init__(self):
        self.values = dict()
        self.isUpdated = False

    def update(self, **values):
        for key in values.keys():
            if self.values.get(key, "") != values.get(key, ""):
                self.isUpdated = True
                self.values[key] = values[key]

    def get(self, k, default):
        return self.values.get(k, default)

    def is_updated(self):
        return self.isUpdated



from subprocess import call
import cv2
from cv2 import VideoCapture


def safe_format(x):
    if x is None:
        return 0
    return x


class CameraManager:
    def __init__(self, port=0):
        self.port = port
        self.camera = VideoCapture(port)

    def set_camera(self, port):
        if self.port is not port:
            try:
                port = int(port)
                self.camera.release()
                self.camera = VideoCapture(port)
                self.port = port
            except ValueError as e:
                pass

    def set_exposure(self, exposure):
        call(['v4l2-ctl', '--device=/dev/video{}'.format(safe_format(self.port)), '-c',
              'exposure_auto=1', '-c', 'exposure_absolute={}'.format(safe_format(exposure))])

    def get_image(self):
        success, img = self.camera.read()
        if success:
            return cv2.UMat(img)

    def release(self):
        self.camera.release()
